/*
 * Scratchbox Remote Shell
 * Copyright (c) 2003 Timo Savola <timo.savola@movial.fi>
 *
 * sbrsh is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include "types.h"
#include "common.h"

const char *oom = "Out of memory";

/**
 * Sets a file descriptor (non-)blocking.
 * @param fd the behaviour of this file descriptor will be changed
 * @param nonblock if TRUE, fd is set non-blocking; if FALSE, fd is set blocking
 */
int set_nonblocking(int fd,
		    bool_t nonblock)
{
	int val;

	val = fcntl(fd, F_GETFL, 0);
	if (val < 0) {
		return -1;
	}

	if (nonblock) {
		if (val & O_NONBLOCK) {
			return 0;
		}

		val |= O_NONBLOCK;
	} else {
		if (!(val & O_NONBLOCK)) {
			return 0;
		}

		val &= ~O_NONBLOCK;
	}

	return fcntl(fd, F_SETFL, val);
}

/**
 * Sets a file descriptor to close-on-exec.
 */
int set_closeonexec(int fd)
{
	int val;

	val = fcntl(fd, F_GETFD, 0);
	if (val < 0) {
		return -1;
	}

	if (val & FD_CLOEXEC) {
		return 0;
	}

	return fcntl(fd, F_SETFD, val | FD_CLOEXEC);
}

int setsockopt_bool(int s,
		    int level,
		    int sockopt,
		    bool_t value)
{
	return setsockopt(s, level, sockopt, &value, sizeof (value));
}

/**
 * Gets a neat progname.
 */
char *get_progname(char *name)
{
	char *p;

	p = strrchr(name, '/');
	if (p && strlen(p) >= 2) {
		return &p[1];
	} else {
		return name;
	}
}

/**
 * Frees all components of a null-terminated vector and the vector itself.
 * @param vec tor
 */
void free_vec(void **vec)
{
	void **p;

	for (p = vec; *p; p++) {
		free(*p);
	}

	free(vec);
}

ssize_t read_line(FILE *file,
		  char *buf,
		  size_t size)
{
	bool_t comm = FALSE;
	size_t i = 0;
	int c;

	if (feof(file)) {
		return -1;
	}

	while (1) {
		c = fgetc(file);
		if (c == EOF || c == '\n') {
			break;
		}
		if (c == '#') {
			comm = TRUE;
		}
		if (!comm && i < size - 1) {
			*buf++ = c;
			i++;
		}
	}
	*buf = '\0';

	return i;
}

char *skip_spaces(char *buf)
{
	while (*buf && isspace(*buf)) {
		buf++;
	}

	return buf;
}

char *find_space(char *buf)
{
	while (*buf && !isspace(*buf)) {
		buf++;
	}

	return buf;
}

char *find_line(char *buf)
{
	while (*buf && *buf++ != '\n');

	return buf;
}

char *trim_string(char *buf)
{
	char *next;

	buf = skip_spaces(buf);

	for (next = buf; 1; ) {
		char *end = find_space(next);

		if (*end == '\0') {
			break;
		}

		next = skip_spaces(end);
		if (*next == '\0') {
			*end = '\0';
			break;
		}
	}

	return buf;
}

int split_string(char *line,
		 ...)
{
	va_list arg;
	char **strp, *str;
	int n = 0;

	va_start(arg, line);

	while (1) {
		strp = va_arg(arg, char **);
		if (!strp) {
			break;
		}

		if (strlen(line) == 0) {
			*strp = NULL;
			continue;
		}

		str = line;

		line = find_space(line);
		if (*line) {
			*line++ = '\0';
			line = skip_spaces(line);
		}

		if (strlen(str) > 0) {
			*strp = str;
			n++;
		} else {
			*strp = NULL;
		}
	}

	va_end(arg);

	return n;
}

size_t string_cat(char *buf,
		  size_t bufsize,
		  const char *piece,
		  ...)
{
	char *str;
	ssize_t avail;
	va_list arg;

	va_start(arg, piece);

	str = buf;
	*str = '\0';

	while (piece) {
		size_t len;

		avail = (bufsize - 1) - strlen(str);
		if (avail <= 0) {
			break;
		}

		len = strlen(piece);
		if (len > avail) {
			strncat(str, piece, avail);
			break;
		}

		strcat(str, piece);

		piece = va_arg(arg, char *);
	}

	va_end(arg);

	return strlen(buf);
}
